<?php

class Lumav_EveryPay_Model_Abstract extends Mage_Core_Model_Abstract
{
    const CURL_METHOD_TYPE_POST = 'POST';
    const CURL_METHOD_TYPE_GET  = 'GET';

    protected $_logFilename   = 'everypay.log';
    protected $_exceptionFile = 'everypay_exception.log';
    protected $_code          = 'lumav_everypay';
    protected $_credentials   = array();
    protected $_helper;

    /**
     * Returns payment gateway URL
     *
     * @return string Gateway URL
     */
    public function getGatewayUrl()
    {
        return Mage::getStoreConfig('payment/' . $this->_code . '/gateway_url');
    }

    /**
     * Return payment processing account
     * @return string Processing account
     */
    public function getProcessingAccount()
    {
        return Mage::getStoreConfig('payment/' . $this->_code . '/account_name');
    }

    /**
     * Return api credentials
     * @return array Api username and secret
     */
    public function getCredentials()
    {
        if (empty($this->_credentials)) {
            if ($this->getGatewayUrl() == Lumav_EveryPay_Model_Source_ApiUrl::$urlList['LIVE']) {
                $username = Mage::getStoreConfig('payment/' . $this->_code . '/api_username');
                $secret   = Mage::getStoreConfig('payment/' . $this->_code . '/api_secret');
            } else {
                $username = Mage::getStoreConfig('payment/' . $this->_code . '/api_username_test');
                $secret   = Mage::getStoreConfig('payment/' . $this->_code . '/api_secret_test');
            }

            $this->_credentials = array(
                'username' => $username,
                'secret'   => $secret,
            );
        }

        return $this->_credentials;
    }

    /**
     * Make curl request
     * @param  string $url    Request url
     * @param  string $method Request type
     * @param  array  $params Request params
     * @return array|bool     Array on success false on curl error
     */
    public function makeRequest($url, $method = self::CURL_METHOD_TYPE_GET, $params = null)
    {
        $credentials = $this->getCredentials();
        $pw          = $credentials['username'] . ':' . $credentials['secret'];

        $options = array(
            CURLOPT_USERPWD        => $pw,
            CURLOPT_HTTPHEADER     => array(
                // "Accept: application/json",
                "Content-Type: application/json",
            ),
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_AUTOREFERER    => true,
            CURLOPT_CONNECTTIMEOUT => 30,
            CURLOPT_TIMEOUT        => 30,
        );

        if ($method == self::CURL_METHOD_TYPE_POST) {
            $options[CURLOPT_POST]       = true;
            $options[CURLOPT_POSTFIELDS] = json_encode($params);
        }

        $ch = curl_init($url);
        curl_setopt_array($ch, $options);

        $result = curl_exec($ch);
        if ($result === false) {
            $this->log(curl_error($ch), 'everypay_curl_error.log');
            curl_close($ch);
            return false;
        }
        curl_close($ch);
        return $result;
    }

    public function getResource()
    {
        return Mage::getSingleton('core/resource');
    }

    public function readAdapter()
    {
        return $this->getResource()->getConnection('core_read');
    }

    public function writeAdapter()
    {
        return $this->getResource()->getConnection('core_write');
    }

    /**
     * Log
     * @param  string $msg      Message
     * @param  string $filename Log file name
     */
    public function log($msg, $filename = null)
    {
        if (!$filename) {
            $filename = $this->_logFilename;
        }

        Mage::log($msg, null, $filename, true);
    }

    /**
     * Log exception
     * @param  object $e
     * @param  string $filename
     */
    public function logException($e, $filename = null)
    {
        if (!$filename) {
            $filename = $this->_exceptionFile;
        }

        Mage::log("\n" . $e->__toString(), Zend_Log::ERR, $filename);
    }
    
    protected function getHelper()
    {
        if(!isset($this->_helper)) {
            $this->_helper = Mage::helper('everypay');
        }
        
        return $this->_helper;
    }
}
