<?php

class Lumav_EveryPay_Model_Api extends Lumav_EveryPay_Model_Abstract
{
    const ONEOFF              = '/payments/oneoff';
    const CIT                 = '/payments/cit';
    const PROCESSING_ACCOUNTS = '/processing_accounts/';
    const PAYMENT_REFERENCE   = '/payments/';

    /**
     * Curl request
     * @param  string $endpoint
     * @param  string $method   request method
     * @param  array  $params   request post data
     * @return json             request return data
     */
    public function request($endpoint, $method = parent::CURL_METHOD_TYPE_GET, $params = null)
    {
        $url = $this->getGatewayUrl() . $endpoint;
        return $this->makeRequest($url, $method, $params);
    }

    /**
     * Returns payment link
     * @param  array  $params  Data used for curl request
     * @param  int    $tokenId Creditcard token id used for saved cards
     * @return string          Payment link for iframe
     */
    public function getPaymentLink($params, $tokenId = null)
    {
        $paymentMethod = $params['payment_method'];

        if ($tokenId && $tokenId != -1 && $paymentMethod === 'card') {
            $params['token'] = $tokenId;
            $result = $this->request(self::CIT, parent::CURL_METHOD_TYPE_POST, $params);

            if ($result === false) {
                Mage::throwException("Payment initialization failed!");
            }
            $result = json_decode($result, true);
            
            return array('url' => $result['payment_link'], 'iframeAllowed' => true);
        }
        $result = $this->request(self::ONEOFF, parent::CURL_METHOD_TYPE_POST, $params);

        if ($result === false) {
            Mage::throwException("Payment initialization failed!");
        }

        $result = json_decode($result, true);
        
        if (isset($result['error']) && !empty($result['error'])) {
            Mage::throwException(sprintf("ERROR: code: %s; message: %s", $result['error']['code'], $result['error']['message']));
        }
        
        foreach ($result['payment_methods'] as $method) {
            if ($method['source'] == $paymentMethod) {
                return array('url' => $method['payment_link'], 'iframeAllowed' => $this->getHelper()->isCardMethod($method));
            }
        }

        return array('url' => $result['payment_link'], 'iframeAllowed' => false);
    }
    
    /**
     * Returns payment data
     * @param  string   $paymentRef Payment reference ID
     * @return array                Payment data
     */
    public function getPaymentData($paymentRef)
    {
        $credentials = $this->getCredentials();
        $endpoint = self::PAYMENT_REFERENCE . $paymentRef . '?api_username=' . $credentials['username'];
        
        $result = $this->request($endpoint, parent::CURL_METHOD_TYPE_GET);

        if ($result === false) {
            Mage::throwException("Payment data initialization failed!");
        }

        $result = json_decode($result, true);
        
        return $result;
    }
}
