<?php

class Lumav_EveryPay_Model_Method extends Lumav_EveryPay_Model_Abstract
{
    const XML_PATH_PAYMENT_METHODS = 'payment/lumav_everypay/payment_methods';

    protected $_api;

    public function _construct()
    {
        $this->_api = Mage::getSingleton('everypay/api');
    }

    /**
     * Update processing account payment methods
     * @return void
     */
    public function updateMethods()
    {
        return $this->_update();
    }

    /**
     * Get method display name
     * @return string Payment method display name
     */
    public function getDisplayName()
    {
        return $this->getData('display_name');
    }

    /**
     * Get available payment methods for processing account
     * @return array
     */
    protected function _getMethods()
    {
        $processingAccount = $this->getProcessingAccount();
        $credentials       = $this->getCredentials();
        $endpoint          = Lumav_EveryPay_Model_Api::PROCESSING_ACCOUNTS . $processingAccount . '?api_username=' . $credentials['username'];

        $result = $this->_api->request($endpoint);
        $result = json_decode($result, true);
        return $result;
    }

    /**
     * Update process gets data ready for insert
     */
    protected function _update()
    {
        $methods = $this->_getMethods();

        if (!$methods) {
            return false;
        }

        $account        = $methods['processing_account_name'];
        $paymentMethods = $methods['payment_methods'];

        $methods = array();

        foreach ($paymentMethods as $method) {
            unset($method['payment_link']);
            $methods[] = $method;
        }
        $methods = serialize($methods);
        Mage::getConfig()->saveConfig(self::XML_PATH_PAYMENT_METHODS, $methods);

        return true;
    }
}
