<?php
class Lumav_EveryPay_EverypayController extends Mage_Core_Controller_Front_Action
{
    /**
     *
     * @var Specifies model to be used to verify response from bank
     */
    protected $_model = 'everypay/everypay';
    
    /**
     *
     * @var Specifies model to be used to make api requests to bank
     */
    protected $_api = 'everypay/api';

    /**
     *
     * @var Specifies payment method code in magento
     */
    protected $_code = 'lumav_everypay';

    /**
     *
     * @var Fieldname of order number in return data (used on automatic response from bank)
     */
    protected $orderNoField = 'order_reference';

    /**
     *
     * @var Fieldname of payment reference in return data (used on automatic response from bank)
     */
    protected $paymentRefField = 'payment_reference';

    /**
     *
     * @var specifies log file name for Payment
     */
    protected $logFile = 'everypay.log';

    /**
     * This action redirects user to bank for payment
     *
     * @return void
     */
    public function redirectAction()
    {
        if (!Mage::getStoreConfig('payment/' . $this->_code . '/active')) {
            return $this->_redirect('checkout/onepage/failure');
        }
        
        $this->loadLayout();
        
        $block = $this->getLayout()->getBlock('everypay');
        $paymentLink = $block->getPaymentLink();
        $useIframe = Mage::getStoreConfig('payment/lumav_everypay/connection_type') && $paymentLink['iframeAllowed'];
        
        if(!empty($paymentLink['url']) && !$useIframe) {
            Mage::app()->getResponse()->setRedirect($paymentLink['url'])->sendResponse();
            return;
        }
        
        $this->renderLayout();
    }

    public function callbackAction()
    {
        if($this->checkReturnParams()) {
            Mage::app()->getResponse()->setHttpResponseCode(200);
            return;
        }
        
        Mage::app()->getResponse()->setHttpResponseCode(403);
        return;
    }

    public function returnAction()
    {
        $this->checkReturnParams();
        
        $session     = Mage::getSingleton('checkout/session');
        $orderNumber = $session->getLastRealOrderId();
        if (!$orderNumber) {
            $this->log('Order number not found in session', __METHOD__, __LINE__);
            $this->_redirect('checkout/onepage/failure');
            return;
        }
        
        $order = Mage::getModel('sales/order')->loadByIncrementId($orderNumber);
        
        //If order has been canceled, redirect to failure page
        if($order->getStatus() == 'canceled') {
            $this->_redirect('checkout/onepage/failure');
            return;
        }
        
        //If order has been paid, redirect to success page
        if($order->hasInvoices()) {
            $this->_redirect('checkout/onepage/success');
            return;
        }
        
        //If the order has not been canceled or paid, load the waiting page
        $this->loadLayout();
        $this->renderLayout();
    }
    
    protected function checkReturnParams()
    {
        $params = array();
        
        $content = $this->getRequest()->getRawBody();
        parse_str($content, $params);
        
        if(empty($params)) {
            $params = $this->getRequest()->getParams();
        }
        $this->log($params, __METHOD__, __LINE__);

        $orderNumber = $params[$this->orderNoField];
        $paymentRef = $params[$this->paymentRefField];
        if (!$orderNumber || !$paymentRef) {
            return false;
        }
        
        $order = Mage::getModel('sales/order')->loadByIncrementId($orderNumber);
        
        if($order->canInvoice()) {
            try {
                $paymentData = $this->getPaymentData($paymentRef);
                $this->dealWithIt($paymentData, $orderNumber, true);
            } catch (Exception $e) {
                Mage::log($e->getMessage(), null, $this->logFile);
            }
        }
        
        return true;
    }
    
    public function statusAction()
    {
        $this->checkReturnParams();
        
        $result = array();
        
        $session     = Mage::getSingleton('checkout/session');
        $orderNumber = $session->getLastRealOrderId();
        if (!$orderNumber) {
            $this->log('Order number not found in session', __METHOD__, __LINE__);
            $result['redirect'] = Mage::getModel('core/url')->getUrl('checkout/onepage/failure');
        } else {        
            $order = Mage::getModel('sales/order')->loadByIncrementId($orderNumber);
            
            //If order has been canceled, redirect to failure page
            if($order->getStatus() == 'canceled') {
                $result['redirect'] = Mage::getModel('core/url')->getUrl('checkout/onepage/failure');
            }
            
            //If order has been paid, redirect to success page
            if($order->hasInvoices()) {
                $result['redirect'] = Mage::getModel('core/url')->getUrl('checkout/onepage/success');
            }
        }
        
        $this->getResponse()->clearHeaders()->setHeader('Content-type','application/json', true);
        $this->getResponse()->setBody(Mage::helper('core')->jsonEncode($result));
    }
    
    public function cancelAction()
    {
        $session     = Mage::getSingleton('checkout/session');
        $orderNumber = $session->getLastRealOrderId();
        if ($orderNumber) {
            $order = Mage::getModel('sales/order')->loadByIncrementId($orderNumber);
            $order->cancel()->save();
            Mage::helper('everypay')->restoreQuote($order);
        }
        $this->_redirect('checkout/onepage/index');
        return;
    }

    protected function dealWithIt($params, $orderNumber, $isCallback = false)
    {
        $model = Mage::getModel($this->_model);
        $model->setOrderNumber($orderNumber);
        $verify = $model->verify($params);
        switch ($verify) {
            case Lumav_EveryPay_Model_Everypay::_VERIFY_SUCCESS:
                if ($isCallback) {
                    $order      = Mage::getModel('sales/order')->loadByIncrementId($orderNumber);
                    $customerId = $order->getCustomerId();
                } else {
                    $customerId = Mage::getSingleton('customer/session')->getCustomerId();
                }

                if ($customerId && isset($params['cc_details']['token'])) {
                    $ccData = $params['cc_details'];
                    // number of customer card tokens
                    $tokenCount = Mage::getResourceModel('everypay/token_collection')
                        ->addFieldToFilter('customer_id', $customerId)
                        ->getSize();

                    // check if card token already exists
                    if (Mage::getResourceModel('everypay/token_collection')
                        ->addFieldToFilter('customer_id', $customerId)
                        ->addFieldToFilter('cc_token', $ccData['token'])
                        ->getSize() == 0) {
                        Mage::getModel('everypay/token')
                            ->setCustomerId($customerId)
                            ->setCcToken($ccData['token'])
                            ->setCcLastFourDigits($ccData['last_four_digits'])
                            ->setCcYear($ccData['year'])
                            ->setCcMonth($ccData['month'])
                            ->setCcType($ccData['type'])
                            ->setIsDefault($tokenCount ? 0 : 1) // set first one to default
                            ->save();
                    }
                }

                /* Send order confirmation */
                if (Mage::getStoreConfig('payment/' . $this->_code . '/order_confirmation') == '1') {
                    try {
                        $order = Mage::getModel('sales/order');
                        $order->load(
                            Mage::getSingleton('checkout/session')->getLastOrderId()
                        );
                        $order->sendNewOrderEmail();
                        $order->save();
                    } catch (Exception $e) {
                        Mage::log(sprintf('%s(%s): %s', __METHOD__, __LINE__, print_r($e->getMessage(), true)), null, $this->logFile);
                    }
                }

                $model->createInvoice();
                break;
            case Lumav_EveryPay_Model_Everypay::_VERIFY_CANCEL:
                $order = Mage::getModel('sales/order')->loadByIncrementId($orderNumber);
                $order->cancel()->save();
                Mage::helper('everypay')->restoreQuote($order);
                break;
        }
    }
    
    protected function getPaymentData($paymentRef)
    {
        $api = Mage::getModel($this->_api);
        return $api->getPaymentData($paymentRef);
    }

    protected function log($txt, $method = __METHOD__, $line = __LINE__)
    {
        Mage::log(sprintf('%s(%s)@%s: %s', $method, $line, $_SERVER['REMOTE_ADDR'], print_r($txt, true)), null, $this->logFile);
    }
}
